<?php
/*
Plugin Name: Keyword to URL
Plugin URI: http://mcngraphics.com/theLab/keyword-to-url/
Description: The plugin allows administrators to build and save a table of keywords and URLs. Once saved any post (or page) that has the keyword will now show that word (or phrase) as a link.
Version: 1.2
Author: mcngraphics.com
Author URI: http://mcngraphics.com
License: GPL

Copyright 2014 mcngraphics.com (email : info@mcngraphics.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as 
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//-------------------------------------------------
//Filter

add_filter( 'the_content', 'keyword_to_url_filter_handler' );
//add_filter( 'the_excerpt', 'keyword_to_url_filter_handler' );

function keyword_to_url_filter_handler( $content ) {
	
	//---------------------------------------------
	//If the special comment is present then exit
	
	//Users can add this comment anywhere in a post to signal not to use the plugin for that post
	if ( stripos( $content, '<!-- no keywords -->' ) !== false ) { return $content; }

	//---------------------------------------------
	//Get the keyword/url list from database
	
	$mcn_keyword_to_url_serialized = get_option('mcn_keyword_to_url_serialized');
	$mcn_keyword_to_url_first_only = get_option('mcn_keyword_to_url_first_only');
	
	//If there are no keywords then exit
	if ( strlen( $mcn_keyword_to_url_serialized ) == 0 ) { return $content; }
	
	//De-serilaize keywords
	$aKeywords = array();
	$s = explode( 'mcn1', $mcn_keyword_to_url_serialized );
	for ( $i=0; $i<count( $s ) - 1; $i++ ) {
		array_push( $aKeywords, explode( 'mcn0', $s[$i] ) );
	}
	
	//---------------------------------------------
	//Find all pre-existing selectors
	
	$aSelector = array();
	$aDebug = array();
	$i = 0;
	$startSelector = 0;
	$endSelector = 0;
	
	while( $i < strlen( $content ) ) {
		
		//We have found a selector ('<' followed by a non-space char)
		if ( ( substr($content, $i, 1) == '<' ) && ( substr($content, $i+1, 1) != ' ' ) ) {
			$startSelector = $i;							
			
			//Selectors and their contents that we want to disregard
			//We want to mark the selector and its content
			if ( substr($content, $i, 3) == '<a ' ) {			
				while( $i < strlen( $content ) ) {				
					if (substr($content, $i, 4) == '</a>') {	
						$endSelector = $i + 4;
							array_push( $aSelector, array( $startSelector, $endSelector ) );
						break;
					}
					$i++;
				}
			} else if ( (substr($content, $i, 2) == '<h') && (is_numeric(substr($content, $i+2, 1))) ) {			
				while( $i < strlen( $content ) ) {				
					if (substr($content, $i, 3) == '</h') {	
						$endSelector = $i + 3;
							array_push( $aSelector, array( $startSelector, $endSelector ) );
						break;
					}
					$i++;
				}
			} else {
			//Selectors that we want to disregard only their arguments
			//We only want to mark the selector, not its content
			//Example: for '<p class="test">Test</p>' we only mark '<p class="test">' and '</p>', not 'Test'
				while( $i < strlen( $content ) ) {				
					if ( substr($content, $i, 1) == '>') {		
						$endSelector = $i+1;
						array_push( $aSelector, array( $startSelector, $endSelector ) );
						break;
					}
					$i++;
				}
			}
		}
		$i++;
	}
	
	/*
	//Debug
	for ( $i=0; $i<count($aSelector); $i++ ) {
		array_push( $aDebug, substr($content, $aSelector[$i][0], $aSelector[$i][1] - $aSelector[$i][0]) );
	}
	var_dump($aDebug);
	*/
	
	//---------------------------------------------
	//Find all the keyword boundaries
	
	$aReplace = array();
	for ( $i=0; $i<count($aKeywords); $i++ ) {
		preg_match_all('/\b' . $aKeywords[$i][0] .'\b/i', $content, $matches, PREG_OFFSET_CAPTURE);
		$aMatches = $matches[0];
		for ( $j=0; $j<count($aMatches); $j++ ) {
			$m = $aMatches[$j][1]; 
			$n = $m + strlen( $aMatches[$j][0] ) - 1;
			$inAnchor = 0;
			for ( $k=0; $k<count($aSelector); $k++ ) {
				$startAnchor = $aSelector[$k][0];
				$closeAnchor = $aSelector[$k][1];
				if ( ( $m > $startAnchor ) && ( $n < $closeAnchor ) ) { $inAnchor = 1;  break; }
			}
			if ( !$inAnchor ) {
				array_push( $aReplace, array( $m, $n, $aMatches[$j][0], $aKeywords[$i][1] ) );
				if ( $mcn_keyword_to_url_first_only == 1 ) { break; } 
			}
		}
	}
	usort($aReplace, 'compareOrder');
	
	//---------------------------------------------
	//Replace keywords with their URLs

	$i = 0;
	$temp = '';
	for ( $j = 0; $j<count($aReplace); $j++ ) {
		$keywordStart = $aReplace[$j][0];
		if ( $keywordStart > $i ) {
			$keywordEnd = $aReplace[$j][1];
			$keyword = $aReplace[$j][2];
			$url = $aReplace[$j][3];
			$temp .= substr( $content, $i, $keywordStart - $i );
			$urlSuffix = '';
			if ( strpos( $url, '@' ) === false ) {
				$urlSuffix = 'http://';
			} else {
				$urlSuffix = 'mailto:';
			}
			$temp .= '<a href="' . $urlSuffix . $url . '">' . $keyword . '</a>';
			$i = $keywordEnd + 1;
		}
	}
	
	$temp .= substr( $content, $i );
	return ( $temp );
}

//---------------------------------------------
//Utilities

//Reoreder the $aReplace array by keywordStart, -KeywordEnd
//This way we order by keyword start and then by the largest keyword that starts there
function compareOrder( $a, $b ) {
	$retval = $a[0] - $b[0];
	if( !$retval ) { 
		return $b[1] - $a[1];
	}
	return $retval;

}

//-------------------------------------------------
//Admin page

if ( is_admin() ){

	add_action( 'admin_init', 'add_keyword_to_url_admin_init' );
	add_action( 'admin_menu', 'add_keyword_to_url_admin_menu' );

	function add_keyword_to_url_admin_init() {
		wp_enqueue_script( 
			'add_keyword_to_url_script',  
			plugins_url( '/keyword-to-url.js', __FILE__ ), 
			false, 
			'1.1' );
	}

	function add_keyword_to_url_admin_menu() {
		//Note if the name had spaces then be sure to use dashes instead of spaces in the slug!
		add_options_page(
			'Keyword to URL', 'Keyword to URL', 'administrator',
			'keyword-to-url', 'keyword_to_url_html_page' );
	}
}

function keyword_to_url_html_page() {
?>
<div>
	<h2>Keyword to URL Options</h2>
	
	<p>Keyword to URL creates a list of keywords with corresponding URLs. 
	<br />Any keyword that appears in your posts (and pages) will show up as a link.
	<br /><strong>Be sure to Save Changes before leaving this page.</strong></p>
	
	<p>
	Keyword: <input id="tKeyword" type="text" />
	URL: <input id="tURL" type="text" />
	<input id="bSave" type="button" value="Add" onclick="mcngraphics.add_my_row()" />
	<span id="tValid1" style="color:#f00; display:none;">* Please enter a Keyword and a URL</span>
	<span id="tValid2" style="color:#f00; display:none;">* Keyword is already in the table</span>
	<span id="tValid3" style="color:#f00; display:none;">* Keyword can not contain quotes</span>
	</p>
	
	<table id="tb">
	<?php
	$mcn_keyword_to_url_serialized = get_option( 'mcn_keyword_to_url_serialized' );
	if ( strlen( $mcn_keyword_to_url_serialized ) > 0 ) { 
		$e1 = explode( 'mcn1', $mcn_keyword_to_url_serialized );
		for ( $i=0; $i<count($e1)-1; $i++ ) {
			$e2 = explode( 'mcn0', $e1[$i] );
			print(
				'<tr>' .
					'<td><a href="http://' . $e2[1] . '">' . $e2[0] . '</a></td>' .
					'<td>' . $e2[1] . '</td>' .
					'<td>' . 
						'<a href="#" onclick="mcngraphics.delete_my_row( this.parentNode.parentNode.rowIndex )">Remove</a>' .
					'</td>' .
				'</tr>');
		}
	}
	?>
	</table>
	
	<form method="post" action="options.php">
		<?php wp_nonce_field( 'update-options' ); ?>
		
		<input 
			id="mcn_keyword_to_url_serialized" name="mcn_keyword_to_url_serialized"
			type="hidden"
			value='<?php echo get_option( 'mcn_keyword_to_url_serialized' ); ?>' />
		
		<br />
		<input 
			id="mcn_keyword_to_url_first_only" name="mcn_keyword_to_url_first_only"
			type="checkbox" 
			value="1" <?php checked( get_option( 'mcn_keyword_to_url_first_only' ), 1 ); ?> />
		Just link the first occurrence of the keyword in the post (otherwise all occurrences are linked).
		
		<input type="hidden" name="action" value="update" />
		<input type="hidden" name="page_options" value="mcn_keyword_to_url_serialized, mcn_keyword_to_url_first_only" />
		<p><input type="submit" value="<?php _e( 'Save Changes' ) ?>" /></p>
	</form>
</div>
<?php
}
?>